package sun.awt.image;

import java.security.AccessController;
import java.security.PrivilegedAction;
import java.awt.image.BufferedImageOp;
import java.awt.image.LookupTable;
import java.awt.image.ConvolveOp;
import java.awt.image.AffineTransformOp;
import java.awt.image.ByteLookupTable;
import java.awt.image.LookupOp;
import java.awt.image.WritableRaster;
import java.awt.image.RasterOp;
import java.awt.image.Kernel;
import java.awt.image.Raster;
import java.awt.image.BufferedImage;

public class ImagingLib
{
    static boolean useLib;
    static boolean verbose;
    private static final int NUM_NATIVE_OPS = 3;
    private static final int LOOKUP_OP = 0;
    private static final int AFFINE_OP = 1;
    private static final int CONVOLVE_OP = 2;
    private static Class[] nativeOpClass;
    
    public ImagingLib() {
    }
    
    private static native boolean init();
    
    public static native int transformBI(BufferedImage p0, BufferedImage p1, double[] p2, int p3);
    
    public static native int transformRaster(Raster p0, Raster p1, double[] p2, int p3);
    
    public static native int convolveBI(BufferedImage p0, BufferedImage p1, Kernel p2, int p3);
    
    public static native int convolveRaster(Raster p0, Raster p1, Kernel p2, int p3);
    
    public static native int lookupByteBI(BufferedImage p0, BufferedImage p1, byte[][] p2);
    
    public static native int lookupByteRaster(Raster p0, Raster p1, byte[][] p2);
    
    private static int getNativeOpIndex(Class clazz) {
        int n = -1;
        int i = 0;
        while (i < 3) {
            if (clazz == ImagingLib.nativeOpClass[i]) {
                n = i;
                break;
            }
            ++i;
        }
        return n;
    }
    
    public static WritableRaster filter(RasterOp rasterOp, Raster raster, WritableRaster compatibleDestRaster) {
        if (!ImagingLib.useLib) {
            return null;
        }
        if (compatibleDestRaster == null) {
            compatibleDestRaster = rasterOp.createCompatibleDestRaster(raster);
        }
        WritableRaster writableRaster = null;
        switch (getNativeOpIndex(rasterOp.getClass())) {
            case 0: {
                LookupTable table = ((LookupOp)rasterOp).getTable();
                if (table.getOffset() != 0) {
                    return null;
                }
                if ((table instanceof ByteLookupTable)) {
                    if (lookupByteRaster(raster, compatibleDestRaster, ((ByteLookupTable)table).getTable()) > 0) {
                        writableRaster = compatibleDestRaster;
                    }
                    break;
                }
                else {
                    break;
                }
            }
            case 1: {
                AffineTransformOp affineTransformOp = (AffineTransformOp)rasterOp;
                double[] flatmatrix = new double[6];
                affineTransformOp.getTransform().getMatrix(flatmatrix);
                if (transformRaster(raster, compatibleDestRaster, flatmatrix, affineTransformOp.getInterpolationType()) > 0) {
                    writableRaster = compatibleDestRaster;
                    break;
                }
                break;
            }
            case 2: {
                ConvolveOp convolveOp = (ConvolveOp)rasterOp;
                if (convolveRaster(raster, compatibleDestRaster, convolveOp.getKernel(), convolveOp.getEdgeCondition()) > 0) {
                    writableRaster = compatibleDestRaster;
                    break;
                }
                break;
            }
        }
        if (writableRaster != null) {
            SunWritableRaster.markDirty(writableRaster);
            return writableRaster;
        }
        return writableRaster;
    }
    
    public static BufferedImage filter(BufferedImageOp obj, BufferedImage obj2, BufferedImage compatibleDestImage) {
        if (ImagingLib.verbose) {
            System.out.println("in filter and op is " + obj + "bufimage is " + obj2 + " and " + compatibleDestImage);
        }
        if (!ImagingLib.useLib) {
            return null;
        }
        if (compatibleDestImage == null) {
            compatibleDestImage = obj.createCompatibleDestImage(obj2, null);
        }
        BufferedImage bufferedImage = null;
        switch (getNativeOpIndex(obj.getClass())) {
            case 0: {
                LookupTable table = ((LookupOp)obj).getTable();
                if (table.getOffset() != 0) {
                    return null;
                }
                if ((table instanceof ByteLookupTable)) {
                    if (lookupByteBI(obj2, compatibleDestImage, ((ByteLookupTable)table).getTable()) > 0) {
                        bufferedImage = compatibleDestImage;
                    }
                    break;
                }
                else {
                    break;
                }
            }
            case 1: {
                AffineTransformOp affineTransformOp = (AffineTransformOp)obj;
                double[] flatmatrix = new double[6];
                affineTransformOp.getTransform();
                affineTransformOp.getTransform().getMatrix(flatmatrix);
                if (transformBI(obj2, compatibleDestImage, flatmatrix, affineTransformOp.getInterpolationType()) > 0) {
                    bufferedImage = compatibleDestImage;
                    break;
                }
                break;
            }
            case 2: {
                ConvolveOp convolveOp = (ConvolveOp)obj;
                if (convolveBI(obj2, compatibleDestImage, convolveOp.getKernel(), convolveOp.getEdgeCondition()) > 0) {
                    bufferedImage = compatibleDestImage;
                    break;
                }
                break;
            }
        }
        if (bufferedImage != null) {
            SunWritableRaster.markDirty(bufferedImage);
            return bufferedImage;
        }
        return bufferedImage;
    }
    
    static boolean access$000() {
        return init();
    }
    
    static {
        ImagingLib.useLib = true;
        ImagingLib.verbose = false;
        ImagingLib.nativeOpClass = new Class[3];
        ImagingLib.useLib = (boolean)AccessController.doPrivileged(new PrivilegedAction<Boolean>() {
            @Override
            public Boolean run() {
                String property = System.getProperty("os.arch");
                if (property != null && property.startsWith("sparc")) {
                    return Boolean.valueOf(ImagingLib.access$000());
                }
                try {
                    System.loadLibrary("mlib_image");
                }
                catch (UnsatisfiedLinkError unsatisfiedLinkError) {
                    return Boolean.FALSE;
                }
                return Boolean.valueOf(ImagingLib.access$000());
            }
        });
        try {
            ImagingLib.nativeOpClass[0] = Class.forName("java.awt.image.LookupOp");
        }
        catch (ClassNotFoundException obj) {
            System.err.println("Could not find class: " + obj);
        }
        try {
            ImagingLib.nativeOpClass[1] = Class.forName("java.awt.image.AffineTransformOp");
        }
        catch (ClassNotFoundException obj2) {
            System.err.println("Could not find class: " + obj2);
        }
        try {
            ImagingLib.nativeOpClass[2] = Class.forName("java.awt.image.ConvolveOp");
        }
        catch (ClassNotFoundException obj3) {
            System.err.println("Could not find class: " + obj3);
        }
    }
}
